unit FileDeleter;

interface
uses
  classes, SysUtils, controls;

type
  TFileDeleter = class
  public
    constructor Create();
    function addFileNameMask(mask : string) : integer;
    function addDirectory(dir : string) : integer;
    procedure setRecursiveDirs(doRecursive : boolean);
    function delFileNameMask(mask : string) : integer;
    function delDirectory(dir : string) : integer;
    procedure setPreviewMode(doPreview : boolean);

    procedure delFiles(deletedFilesList : TStrings);

    function clearAllFileNameMasks : integer;
    function clearAllDirectories : integer;
    procedure clearAll;


//get methods
    function getPreviewMode: boolean;
    function getRecursiveDirs : boolean;
    procedure getDirectories(dirs : TStrings);
    procedure getFileNameMasks(masks : TStrings);


//methods to deal with saved settings
    function setParamsFileName(fileName : string) : integer;
    function loadParamsFromFile : integer;
    function saveParamsToFile : integer;


//version 1.0.3

    procedure setUseAdvanced(use : boolean);
    function getUseAdvanced : boolean;
    procedure setUseFromDate(use : boolean);
    function getUseFromDate : boolean;
    procedure setUseToDate(use : boolean);
    function getUseToDate : boolean;
    procedure setFromDate(FromDate : TDate);
    procedure setFromTime(FromTime : TTime);
    procedure setToDate(ToDate : TDate);
    procedure setToTime(ToTime : TTime);
    function getFromDate : TDate;
    function getFromTime : TTime;
    function getToDate : TDate;
    function getToTime : TTime;

  private
    previewMode, recSubDirs : boolean;
    paramsFileName : string;

    dirList, maskList : TStringList;
    procedure getSubDirs(startDir : string; subDirList : TStrings);

    procedure localDelFiles(dir, mask : string; deletedFilesList : TStrings);
    procedure localDelFilesBase(dir, mask : string; deletedFilesList : TStrings);
    function isValidMask(mask : string) : boolean;
    procedure resetToDefault;

//new version 1.0.3
  private
    useAdvanced, useFromDate, useToDate : boolean;
    fDate, tDate : TDate;
    fTime, tTime : TTime;

    procedure resetAdvancedToDefault;
    function satisfyAdvanced(f : TSearchRec) : boolean;
    function getTimeOnly(dt : TDateTime) : TTime;
    function getDateOnly(dt : TDateTime) : TDate;
    function getFileInfo(f : TSearchRec; bTime : boolean; bSize : boolean) : string;
    function loadAdvancedParamsFromFile(fl : TStringList) : integer;
    function myStrToDateTime(s : string) : TDateTime;
  end;
implementation

uses
  FileCtrl;


  const
    paramsDirsListTitle = '[Directories]';
    paramsMasksListTitle = '[FileNameMasks]';
    paramsOptionsTitle = '[Options]';
    paramsRecursiveDirs = 'RecursiveDirs';
    paramsPreviewMode = 'PreviewMode';

    paramsFirstLine = 'This is a generated file - Please DO NOT Edit!';

    paramsAdvancedOptions = '[Advanced Options]';
    paramsUseFromDate = 'UseFromDate';
    paramsUseToDate = 'UseToDate';
    paramsFromDate = 'FromDate';
    paramsFromTime = 'FromTime';
    paramsToDate = 'ToDate';
    paramsToTime = 'ToTime';



///////public

function TFileDeleter.addDirectory(dir: string): integer;
begin
  if not DirectoryExists(dir) then
  begin
    Result := 1;
    exit;
  end;
  if dirList.IndexOf(dir) > -1 then
  begin
    Result := 2;
    exit;
  end;

  dirList.Add(dir);
  Result := 0;
end;

function TFileDeleter.addFileNameMask(mask: string): integer;
begin
  Result := 1;
  if isValidMask(mask) then
  begin
    maskList.Add(mask);
    Result := 0;
  end;
end;

function TFileDeleter.delDirectory(dir: string): integer;
begin
  Result := 1;
  if dirList.IndexOf(dir) > -1 then
  begin
    dirList.Delete(dirList.IndexOf(dir));
    Result := 0;
  end;
end;

function TFileDeleter.delFileNameMask(mask: string): integer;
begin
  Result := 1;
  if maskList.IndexOf(mask) > -1 then
  begin
    maskList.Delete(maskList.IndexOf(mask));
    Result := 0;
  end;

end;

procedure TFileDeleter.setRecursiveDirs(doRecursive: boolean);
begin
  recSubDirs := doRecursive;
end;

constructor TFileDeleter.Create;
begin
  inherited;
  resetToDefault;
end;

procedure TFileDeleter.delFiles(deletedFilesList: TStrings);
var
  i, j : integer;
begin
  deletedFilesList.Clear;
  if (dirList.Count > 0) and (maskList.Count > 0) then
  for i := 0 to dirList.Count -1 do
  begin
    for j := 0 to maskList.Count -1 do
    begin
      localDelFiles(dirList[i], maskList[j], deletedFilesList);
    end;
  end;
end;


procedure TFileDeleter.setPreviewMode(doPreview: boolean);
begin
  previewMode := doPreview;
end;

///////public - end

procedure TFileDeleter.localDelFiles(dir, mask: string;
  deletedFilesList: TStrings);
var
  subDirs : TStringList;
  i : integer;
begin
  if not recSubDirs then
    localDelFilesBase(dir, mask, deletedFilesList)
  else
  begin
    localDelFilesBase(dir, mask, deletedFilesList);  //always the current directory

    subDirs := TStringList.Create;
    getSubDirs(dir, subDirs);
    for i := 0 to subDirs.Count -1 do
    begin
      localDelFiles(subDirs[i], mask, deletedFilesList);
    end;
  end;
end;

procedure TFileDeleter.getSubDirs(startDir: string; subDirList: TStrings);
var
  sr: TSearchRec;
  i, ii : integer;
begin
  try
    subDirList.Clear;
    if FindFirst(startDir + '\*.', faDirectory, sr) = 0 then //something found
    repeat
      subDirList.Add(startDir + '\' + sr.Name);
    until FindNext(sr) <> 0;

//remove . e .. directories
    i := subDirList.IndexOf(startDir + '\' + '.');
    if i > -1 then
      subDirList.Delete(i);
    ii := subDirList.IndexOf(startDir + '\' + '..');
    if ii > -1 then
      subDirList.Delete(ii);
  finally
    FindClose(sr);
  end;
end;

procedure TFileDeleter.localDelFilesBase(dir, mask: string;
  deletedFilesList: TStrings);
{the actual method performing the deletion on a single dir
and with a single file mask.}

var
  sr: TSearchRec;
  bTime, bSize : boolean;
begin
  bTime := False;
  bSize := False;
  if getUseAdvanced then
  begin
    bTime := True;
    bSize := True;
  end;

  try
    if FindFirst(dir + '\' + mask, 0, sr) = 0 then //something found
    repeat
      if getUseAdvanced and (not satisfyAdvanced(sr)) then
        Continue;
      if previewMode then
//        deletedFilesList.Add(dir + '\' + sr.Name)
        deletedFilesList.Add(dir + '\' + getFileInfo(sr, bTime, bSize))
      else if DeleteFile(dir + '\' + sr.Name) then
//        deletedFilesList.Add(dir + '\' + sr.Name)
        deletedFilesList.Add(dir + '\' + getFileInfo(sr, bTime, bSize))
      else
        deletedFilesList.Add('NOT DELETED: ' + dir + '\' + getFileInfo(sr, bTime, bSize));
    until FindNext(sr) <> 0;
  finally
    FindClose(sr);
  end;
end;



function TFileDeleter.isValidMask(mask: string): boolean;
begin
//TODO : isValidMask : it checks if the mask is correct.
  Result := True;
end;


function TFileDeleter.clearAllDirectories: integer;
begin
  Result := 1;
  if dirList.Count > 0 then
  begin
    dirList.Clear;
    Result := 0;
  end;
end;

function TFileDeleter.clearAllFileNameMasks: integer;
begin
  Result := 1;
  if maskList.Count > 0 then
  begin
    maskList.Clear;
    Result := 0;
  end;
end;

procedure TFileDeleter.resetToDefault;
begin
  if not Assigned(maskList) then
    maskList := TStringList.Create()
  else
    clearAllFileNameMasks;

  if not Assigned(dirList) then
    dirList := TStringList.Create()
  else
    clearAllDirectories;

  setRecursiveDirs(False);
  setPreviewMode(False);
  paramsFileName := '';
  setUseAdvanced(False);
  resetAdvancedToDefault;
end;

function TFileDeleter.loadParamsFromFile: integer;
var
  dirs, masks, fileLines : TStringList;
  f : TextFile;
  s : string;
  i : integer;
  recDirs, prevMode : integer;
begin
  if (paramsFileName <> '') and (FileExists(paramsFileName)) then
  begin
    dirs := TStringList.Create;
    masks := TStringList.Create;
    fileLines := TStringList.Create;

    //read the file into fileLines
    AssignFile(f, paramsFileName);
    Reset(f);
    while not eof(f) do
    begin
      Readln(f, s);
      fileLines.Add(s);
    end;
    CloseFile(f);

    //search for the directories list
    i := fileLines.IndexOf(paramsDirsListTitle);
    if i <> -1 then
    begin
      Inc(i);//next item
      repeat
        if DirectoryExists(fileLines.Strings[i]) then
          dirs.Add(fileLines.Strings[i])
        else
        begin
          dirs.Clear;
          break;
        end;
        Inc(i);
      until (fileLines.Count < i) OR (fileLines.Strings[i] = '') OR
        (fileLines.Strings[i] = paramsMasksListTitle);
    end;

    //search for the fileName masks list
    i := fileLines.IndexOf(paramsMasksListTitle);
    if i <> -1 then
    begin
      Inc(i);//next item
      repeat
        if isValidMask(fileLines.Strings[i]) then
          masks.Add(fileLines.Strings[i])
        else
        begin
          masks.Clear;
          break;
        end;
        Inc(i);
      until (fileLines.Count < i) OR (fileLines.Strings[i] = '') OR
        (fileLines.Strings[i] = paramsOptionsTitle);
    end;

    //search for the options list
    recDirs := 0;
    i := fileLines.IndexOfName(paramsRecursiveDirs);
    if i <> -1 then  //there is paramsRecursiveDirs option
    begin
      recDirs := StrToInt(fileLines.Values[paramsRecursiveDirs]);
    end;

    prevMode := 0;
    i := fileLines.IndexOfName(paramsPreviewMode);
    if i <> -1 then  //there is paramsPreviewMode option
    begin
      prevMode := StrToInt(fileLines.Values[paramsPreviewMode]);
    end;

//advanced options
    loadAdvancedParamsFromFile(fileLines);

    if (dirs.Count = 0) OR (masks.Count = 0) then  //wrong information provided
      Result := 1
    else
    begin
//clear the previous values
      clearAllDirectories;
      dirList.Assign(dirs);
      clearAllFileNameMasks;
      maskList.Assign(masks);
      setRecursiveDirs(recDirs = 1);
      setPreviewMode(prevMode = 1);
      Result := 0;  //ok
    end;
  end
  else
  begin
    Result := 1;
  end;
end;

function TFileDeleter.setParamsFileName(fileName: string): integer;
begin
  paramsFileName := fileName;
  Result := 0;
end;

procedure TFileDeleter.getDirectories(dirs: TStrings);
begin
  dirs.Assign(dirList);
end;

procedure TFileDeleter.getFileNameMasks(masks: TStrings);
begin
  masks.Assign(maskList);
end;

function TFileDeleter.getPreviewMode: boolean;
begin
  Result := previewMode;
end;

function TFileDeleter.getRecursiveDirs: boolean;
begin
  Result := recSubDirs;
end;

function TFileDeleter.saveParamsToFile: integer;
var
  fileLines : TStringList;
  i : integer;
begin
  Result := 1;
  if paramsFileName = '' then
    Exit;
  Result := 2;
  if (dirList.Count = 0) or (maskList.Count = 0) then
    Exit;

  Result := 3;

  fileLines := TStringList.Create;
  fileLines.Add(paramsFirstLine);
  fileLines.Add('Generated on: ' + DateTimeToStr(Date() + Time()));
  fileLines.Add('');
  fileLines.Add(paramsDirsListTitle);

//add the list of dirs:
  for i := 0 to dirList.Count -1 do
    fileLines.Add(dirList.Strings[i]);
  fileLines.Add('');

//add the list of masks:
  fileLines.Add(paramsMasksListTitle);
  for i := 0 to maskList.Count -1 do
    fileLines.Add(maskList.Strings[i]);
  fileLines.Add('');


    //search for the options list
  fileLines.Add(paramsOptionsTitle);
  if getRecursiveDirs then
    fileLines.Add(paramsRecursiveDirs + '=1')
  else
    fileLines.Add(paramsRecursiveDirs + '=0');

  if getPreviewMode then
    fileLines.Add(paramsPreviewMode + '=1')
  else
    fileLines.Add(paramsPreviewMode + '=0');

//v 1.0.3 Use of advanced options
  if useAdvanced then
  begin
    fileLines.Add('');
    fileLines.Add(paramsAdvancedOptions);
    //From date
    if useFromDate then
    begin
      fileLines.Add(paramsUseFromDate + '=1');
      fileLines.Add(paramsFromDate + '=' + DateToStr(getFromDate));
      fileLines.Add(paramsFromTime + '=' + TimeToStr(getFromTime));
    end
    else
      fileLines.Add(paramsUseFromDate + '=0');

    //To date
    if useToDate then
    begin
      fileLines.Add(paramsUseToDate + '=1');
      fileLines.Add(paramsToDate + '=' + DateToStr(getToDate));
      fileLines.Add(paramsToTime + '=' + TimeToStr(getToTime));
    end
    else
      fileLines.Add(paramsUseToDate + '=0');

  end; //useAdvanced

  try
    fileLines.SaveToFile(paramsFileName);
    Result := 0;
  except
  end;
end;

procedure TFileDeleter.setUseAdvanced(use: boolean);
begin
  useAdvanced := use;

//if false -> clear all the adv opt? maybe  
end;

function TFileDeleter.getUseAdvanced: boolean;
begin
  Result := useAdvanced;
end;

function TFileDeleter.getUseFromDate: boolean;
begin
  Result := getUseAdvanced and useFromDate;
end;

function TFileDeleter.getUseToDate: boolean;
begin
  Result := getUseAdvanced and useToDate;
end;

procedure TFileDeleter.setUseFromDate(use: boolean);
begin
  if getUseAdvanced then
    useFromDate := use;
end;

procedure TFileDeleter.setUseToDate(use: boolean);
begin
  if getUseAdvanced then
    useToDate := use;
end;

procedure TFileDeleter.resetAdvancedToDefault;
begin
  if getUseAdvanced then
  begin
    setUseFromDate(False);
    setUseToDate(False);
  end;
end;

procedure TFileDeleter.clearAll;
begin
  resetToDefault;
end;

procedure TFileDeleter.setFromDate(FromDate: TDate);
begin
  if getUseAdvanced and getUseFromDate then
    fDate := getDateOnly(FromDate);
end;

procedure TFileDeleter.setFromTime(FromTime: TTime);
begin
  if getUseAdvanced and getUseFromDate then
    fTime := getTimeOnly(FromTime);
end;

procedure TFileDeleter.setToDate(ToDate: TDate);
begin
  if getUseAdvanced and getUseToDate then
    tDate := getDateOnly(ToDate);
end;

procedure TFileDeleter.setToTime(ToTime: TTime);
begin
  if getUseAdvanced and getUseToDate then
    tTime := getTimeOnly(ToTime);
end;

function TFileDeleter.getFromDate: TDate;
begin
  if getUseAdvanced and getUseFromDate then
    Result := getDateOnly(fDate)
  else
    Result := -1;
end;

function TFileDeleter.getFromTime: TTime;
begin
  if getUseAdvanced and getUseFromDate then
    Result := getTimeOnly(fTime)
  else
    Result := -1;
end;

function TFileDeleter.getToDate: TDate;
begin
  if getUseAdvanced and getUseToDate then
    Result := getDateOnly(tDate)
  else
    Result := -1;
end;

function TFileDeleter.getToTime: TTime;
begin
  if getUseAdvanced and getUseToDate then
    Result := getTimeOnly(tTime)
  else
    Result := -1;
end;

function TFileDeleter.satisfyAdvanced(f: TSearchRec): boolean;
var
  fd : TDateTime;
  ft : TDateTime;
  fdt : TDateTime;

begin
//if an advanced property is not specifed/activated then
//it is satisfied by default

//as soon as a property is not satisfied the routine exits
  Result := False;
  fdt := FileDateToDateTime(f.Time);
  fd := Trunc(fdt);
  ft := fdt - Trunc(fdt);

  if getUseFromDate then
  begin
    if (getFromDate > fd) or ((getFromDate = fd) AND (getFromTime > ft)) then
      exit;
  end;

  if getUseToDate then
  begin
    if (getToDate < fd) or ((getToDate = fd) AND (getToTime < ft)) then
      exit;
  end;

  Result := True;
end;

function TFileDeleter.getDateOnly(dt: TDateTime): TDate;
begin
  Result := Trunc(dt);
end;

function TFileDeleter.getTimeOnly(dt: TDateTime): TTime;
begin
  Result := dt - Trunc(dt);
end;

function TFileDeleter.getFileInfo(f: TSearchRec; bTime,
  bSize: boolean): string;
var
  s, sep : string;
  fdt : TDateTime;
begin
  sep := '-';
  s := f.Name;
  if bTime then
  begin
    fdt := FileDateToDateTime(f.Time);
    s := s +  sep + DateToStr(getDateOnly(fdt)) + sep + TimeToStr(getTimeOnly(fdt));
  end;
  if bSize then
  begin
    s := s + sep + IntToStr(f.Size) + 'b';
  end;
  Result := s;
end;

function TFileDeleter.loadAdvancedParamsFromFile(fl: TStringList): integer;
var
  i, useFD, useTD : integer;
  fd, ft, td, tt : string;
begin
  useFD := 0;
  useTD := 0;
  fd := '';
  ft := '';
  td := '';
  tt := '';

  //advanced options
  i := fl.IndexOf(paramsAdvancedOptions);
  if i <> -1 then  //there is paramsAdvancedOptions
  begin
    i := fl.IndexOfName(paramsUseFromDate);
    if i <> -1 then  //there is paramsUseFromDate
    begin
      useFD := StrToInt(fl.Values[paramsUseFromDate]);
    end;
    if useFD = 1 then //use of From Date activated
    begin
      if fl.IndexOfName(paramsFromDate) <> -1 then
        fd := fl.Values[paramsFromDate];
      if fl.IndexOfName(paramsFromTime) <> -1 then
        ft := fl.Values[paramsFromTime];
    end;

    i := fl.IndexOfName(paramsUseToDate);
    if i <> -1 then  //there is paramsUseToDate
    begin
      useTD := StrToInt(fl.Values[paramsUseToDate]);
    end;
    if useTD = 1 then //use of To Date activated
    begin
      if fl.IndexOfName(paramsToDate) <> -1 then
        td := fl.Values[paramsToDate];
      if fl.IndexOfName(paramsToTime) <> -1 then
        tt := fl.Values[paramsToTime];
    end;
//all the info collected, let's set the values

    setUseAdvanced(True);
    setUseFromDate(useFD = 1);
    setFromDate(myStrToDateTime(fd));
    setFromTime(myStrToDateTime(ft));
    setUseToDate(useTD = 1);
    setToDate(myStrToDateTime(td));
    setToTime(myStrToDateTime(tt));
  end
  else
    setUseAdvanced(False);

  Result := 0;
end;

function TFileDeleter.myStrToDateTime(s: string): TDateTime;
begin
  try
    Result := StrToDateTime(s);
  except
    on e : Exception do
    Result := 0;
  end;
end;

end.
